/* mq - Gnome Music Quiz Game
Copyright (C) 2004 - Charles Schmidt <cschmidt2@emich.edu>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <config.h>
#include <gnome.h>
#include <libgnomeui/gnome-window-icon.h>
#include <string.h>
#include <gst/gst.h>

#include "eel-gconf-extensions.h"
#include "rhythmdb.h"
#include "rhythmdb-query-model.h"
#include "rb-file-helpers.h"
#include "rb-string-helpers.h"
#include "rb-player.h"

#include "mq.h"

/* objects */
static RhythmDB *db = NULL;
static RhythmDBQueryModel *game_model = NULL;
static RBPlayer *player = NULL;

/* app items */
static GtkWidget *db_load_window = NULL;
static GtkWidget *window = NULL;
static GtkLabel *header_label = NULL;
static GtkLabel *round_score_label = NULL;
static GtkButton *game_buttons[5] = {NULL};
static GtkLabel *score_label = NULL;
static GtkLabel *round_label = NULL;


/* game information */
static gboolean db_loaded = FALSE;
static gboolean first_run = TRUE;
static gfloat volume = 0;
static MQGameType game_type = MQ_GAME_ARTIST;
static gint round = 0;
static gint num_players = 0;
static gint current_player = 0;
static gint *scores = NULL;
static gint num_songs_in_model = 0;

/* round information */
static RhythmDBEntry *round_entries[5] = {NULL};
static gint32 round_answer;
static glong starting_point;
static gint round_score;
static guint round_score_start_decrementing_timeout_id = -1;
static guint round_score_decrement_timeout_id = -1;
static guint round_score_label_update_timeout_id = -1;
static guint round_entries_desensitize_timeout_id = -1;
static gboolean round_entries_sensitive[5] = {TRUE};

/* game functions */
static void new_game(void);
static void play_round(void);
static GtkWidget * make_scores_widget(void);

/* timeouts */
static gboolean player_seek_timeout(gpointer data);
static gboolean player_set_volume_timeout(gpointer data);
static gboolean round_score_start_decrementing_timeout(gpointer data);
static gboolean round_score_decrement_timeout(gpointer data);
static gboolean round_score_label_update_timeout(gpointer data);
static gboolean round_entries_desensitize_timeout(gpointer data);
static gboolean db_load_progress_bar_pulse_timeout(GtkProgressBar *pbar);

/* callbacks */
static void db_load_complete_cb(RhythmDB *db,gpointer data);
static void game_model_load_complete_cb(RhythmDBQueryModel *model,gpointer data);
static void game_button_clicked_cb(GtkWidget *game_button,gpointer data);
static void new_game_druid_page_finish_cb(GnomeDruidPage *page,GnomeDruid *druid,gpointer data);
static gboolean new_game_druid_page_cancel_cb(GnomeDruidPage *page,GnomeDruid *druid,gpointer data);

/* menu callbacks */
static void new_game_cb(GtkWidget *widget,gpointer data);
static void scores_cb(GtkWidget *widget,gpointer data);
static void quit_cb(GtkWidget *widget,gpointer data);
static void about_cb(GtkWidget *widget,gpointer data);


static GnomeUIInfo gamemenu[] = {
        GNOMEUIINFO_MENU_NEW_GAME_ITEM(new_game_cb, NULL),
	GNOMEUIINFO_SEPARATOR,
//	GNOMEUIINFO_MENU_SCORES_ITEM(scores_cb, NULL),
//	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_QUIT_ITEM(quit_cb, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo helpmenu[] = {
//        GNOMEUIINFO_HELP("mq"),
	GNOMEUIINFO_MENU_ABOUT_ITEM(about_cb, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo mainmenu[] = {
        GNOMEUIINFO_MENU_GAME_TREE(gamemenu),
        GNOMEUIINFO_MENU_HELP_TREE(helpmenu),
	GNOMEUIINFO_END
};

static void new_game()
{
	GtkWidget *druid_window;
	GtkWidget *druid;
	gchar *filename = NULL;
	GdkPixbuf *pixbuf = NULL;
	GtkWidget *page;
	GtkWidget *box;
	GtkWidget *label;
	GtkWidget *spin_button;
	GtkWidget *radio_button;
	const gchar *texts[] = {_("Artists"),_("Albums"),_("Song Titles")};
	gint i;

	if (scores) {
		g_free(scores);
		scores = NULL;
	}

	if (round_score_start_decrementing_timeout_id != -1) {
		g_source_remove(round_score_start_decrementing_timeout_id);
	}
	if (round_score_decrement_timeout_id != -1) {
		g_source_remove(round_score_decrement_timeout_id);
	}
	if (round_score_label_update_timeout_id != -1) {
		g_source_remove(round_score_label_update_timeout_id);
	}
	if (round_entries_desensitize_timeout_id != -1) {
		g_source_remove(round_entries_desensitize_timeout_id);
	}
	round_entries_desensitize_timeout_id = round_score_start_decrementing_timeout_id = round_score_decrement_timeout_id = round_score_label_update_timeout_id = -1;

	druid = gnome_druid_new_with_window(_("New Music Quiz Game"),GTK_WINDOW(window),TRUE,&druid_window);
	g_object_set_data(G_OBJECT(druid),"window",druid_window);

	filename = gnome_program_locate_file(NULL,GNOME_FILE_DOMAIN_APP_PIXMAP,("gnome-mq.png"),TRUE,NULL);

	if (filename != NULL) {
		pixbuf = gdk_pixbuf_new_from_file(filename,NULL);
		g_free (filename);
	}

	/* first page */
	page = gnome_druid_page_edge_new_with_vals(GNOME_EDGE_START,TRUE,_("New Music Quiz Game"),_("This simple wizard will help you set up a new Music Quiz game.  All you need to do is answer a few questions."),pixbuf,NULL,NULL);
	gnome_druid_append_page(GNOME_DRUID(druid),GNOME_DRUID_PAGE(page));
	gtk_widget_show_all(page);
	g_signal_connect(G_OBJECT(page),"cancel",G_CALLBACK(new_game_druid_page_cancel_cb),NULL);

	/* num players page */
	page = gnome_druid_page_standard_new_with_vals(_("Choose Number of Players"),pixbuf,NULL);
	box = gtk_hbox_new(FALSE,6);
	label = gtk_label_new_with_mnemonic(_("_Number of Players:"));
	gtk_box_pack_start(GTK_BOX(box),label,FALSE,FALSE,0);
	spin_button = gtk_spin_button_new_with_range(1,5,1);
	gtk_box_pack_start(GTK_BOX(box),spin_button,FALSE,FALSE,0);
	g_object_set_data(G_OBJECT(druid),"spin_button",spin_button);
	gnome_druid_page_standard_append_item(GNOME_DRUID_PAGE_STANDARD(page),_("How many people will play?"),box,NULL);
	gnome_druid_append_page(GNOME_DRUID(druid),GNOME_DRUID_PAGE(page));
	gtk_widget_show_all(page);
	g_signal_connect(G_OBJECT(page),"cancel",G_CALLBACK(new_game_druid_page_cancel_cb),NULL);

	/* player name page ? */

	/* game type page */
	page = gnome_druid_page_standard_new_with_vals(_("Select Game Type"),pixbuf,NULL);
	box = gtk_vbox_new(FALSE,6);
	radio_button = gtk_radio_button_new_with_mnemonic(NULL,_("Guess _Artist"));
	gtk_box_pack_start(GTK_BOX(box),radio_button,FALSE,FALSE,0);
	radio_button = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(radio_button),_("Guess A_lbum"));
	gtk_box_pack_start(GTK_BOX(box),radio_button,FALSE,FALSE,0);
	radio_button = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(radio_button),_("Guess Song _Title"));
	gtk_box_pack_start(GTK_BOX(box),radio_button,FALSE,FALSE,0);
	g_object_set_data(G_OBJECT(druid),"radio_button_group",gtk_radio_button_get_group(GTK_RADIO_BUTTON(radio_button)));
	gnome_druid_page_standard_append_item(GNOME_DRUID_PAGE_STANDARD(page),_("What kind of game will you play?"),box,NULL);
	gnome_druid_append_page(GNOME_DRUID(druid),GNOME_DRUID_PAGE(page));
	gtk_widget_show_all(page);
	g_signal_connect(G_OBJECT(page),"cancel",G_CALLBACK(new_game_druid_page_cancel_cb),NULL);

	/* last page */
	page = gnome_druid_page_edge_new_with_vals(GNOME_EDGE_FINISH,TRUE,_("Ready to Play"),_("Get ready!  The game will begin when you close this wizard."),pixbuf,NULL,NULL);
	gnome_druid_append_page(GNOME_DRUID(druid),GNOME_DRUID_PAGE(page));
	gtk_widget_show_all(page);
	g_signal_connect(G_OBJECT(page),"cancel",G_CALLBACK(new_game_druid_page_cancel_cb),NULL);
	g_signal_connect(G_OBJECT(page),"finish",G_CALLBACK(new_game_druid_page_finish_cb),NULL);

	if (first_run == TRUE) {
		g_signal_connect(G_OBJECT(druid),"cancel",G_CALLBACK(gtk_main_quit),NULL);
		g_signal_connect(G_OBJECT(druid_window),"delete_event",G_CALLBACK(gtk_main_quit),NULL);
	}
	
	gtk_widget_show_all(druid);

	return;
}

static void play_round()
{
	gint i;
	const gchar *strings[5];
	gint j;
	gint32 path_number;
	GtkTreePath *path;
	GtkTreeIter iter;
	gchar *text;
	glong duration;
	const gchar *uri;
	GError *error = NULL;

	round++;

	if (round > NUM_ROUNDS) {
		GtkWidget *dialog;
		GtkBox *vbox;
		GtkWidget *label;
		GtkWidget *hsep;
		GtkWidget *score_widget;
		gint response;

		gtk_widget_set_sensitive(window,FALSE);

		dialog = gtk_dialog_new_with_buttons(_("Scores"),GTK_WINDOW(window),GTK_DIALOG_MODAL,GTK_STOCK_YES,GTK_RESPONSE_YES,GTK_STOCK_NO,GTK_RESPONSE_NO,NULL);

		vbox = GTK_BOX(GTK_DIALOG(dialog)->vbox);

		label = gtk_label_new(_("Final Scores"));
		gtk_box_pack_start(vbox,label,FALSE,FALSE,0);

		hsep = gtk_hseparator_new();
		gtk_box_pack_start(vbox,hsep,FALSE,FALSE,6);

		score_widget = make_scores_widget();
		gtk_box_pack_start(vbox,score_widget,TRUE,TRUE,0);

		hsep = gtk_hseparator_new();
		gtk_box_pack_start(vbox,hsep,FALSE,FALSE,6);

		label = gtk_label_new(_("Play Again?"));
		gtk_box_pack_start(vbox,label,FALSE,FALSE,0);

		gtk_widget_show_all(dialog);

		response = gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);

		switch (response) {
			case GTK_RESPONSE_NO:
			case GTK_RESPONSE_DELETE_EVENT:
				gtk_main_quit();

				break;
			case GTK_RESPONSE_YES:
				new_game();

				break;
			default:
				break;
		}

		return;
	}

	current_player++;
	current_player = current_player % num_players;
	
	round_answer = g_random_int_range(0,5);
	round_score = 1000;

	for (i = 0; i < 5; i++) {
		gboolean found_good_one = TRUE;

		path_number = g_random_int_range(0,num_songs_in_model-1);
		path = gtk_tree_path_new_from_indices(path_number,-1);
		gtk_tree_model_get_iter(GTK_TREE_MODEL(game_model),&iter,path);
		gtk_tree_path_free(path);

		gtk_tree_model_get(GTK_TREE_MODEL(game_model),&iter,0,&(round_entries[i]),-1);
		strings[i] = rhythmdb_entry_get_string(db,round_entries[i],game_type);
		for (j = 0; j < i; j++) {
			if (strcmp(strings[i],strings[j]) == 0) {
				found_good_one = FALSE;
			}
		}

		if (found_good_one == FALSE) {
			i--;
			continue;
		}

		gtk_button_set_label(game_buttons[i],strings[i]);
		gtk_widget_set_sensitive(GTK_WIDGET(game_buttons[i]),TRUE);
		round_entries_sensitive[i] = TRUE;

		if (i == round_answer) {
			duration = rhythmdb_entry_get_long(db,round_entries[i],RHYTHMDB_PROP_DURATION);

			starting_point = (glong)g_random_int_range(15,duration-15);
			uri = rhythmdb_entry_get_string(db,round_entries[i],RHYTHMDB_PROP_LOCATION);
		}
	}

	gtk_label_set_text(round_score_label,"1000");

	rb_player_open(player,uri,&error);
	// FIXME this should be its own function, like find_playable_media() or something that opens random media until it finds one that works
	if (error) {
		g_warning("error opening %s: %s",uri,error->message);
		g_error_free(error);
		error = NULL;
	}

	rb_player_play(player,&error);

	// FIXME this is really weird.  if you try to seek immediatly, it doesnt work.  if you try to start playing & seek in the timeout it doesnt work.  so, go ahead and play with the volume down and then seek & turn the volume up after a bit
	rb_player_set_volume(player,0);
	g_timeout_add(750,(GSourceFunc)player_seek_timeout,NULL);

	if (error) {
		g_warning("error playing %s:%s",uri,error->message);
		g_error_free(error);
		error = NULL;
	}


	// FIXME how do i do i18n?
	text = g_strdup_printf("Player %d's turn",current_player+1);
	gtk_label_set_text(header_label,text);
	g_free(text);
	text = g_strdup_printf("Score: %d",scores[current_player]);
	gtk_label_set_text(score_label,text);
	g_free(text);
	text = g_strdup_printf("Round: %d/%d",round,NUM_ROUNDS);
	gtk_label_set_text(round_label,text);
	g_free(text);

	return;
}

static GtkWidget * make_scores_widget(void)
{
	GtkWidget *table;
	GtkWidget *label;
	gchar *text;
	gint i;

	table = gtk_table_new(num_players+1,2,TRUE);

	label = gtk_label_new(_("Name"));
	gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,0,1);

	label = gtk_label_new(_("Score"));
	gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,0,1);

	for (i = 0; i < num_players; i++) {
		text = g_strdup_printf("Player %d",i+1);
		label = gtk_label_new(text);
		g_free(text);
		gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,i+1,i+2);

		text = g_strdup_printf("%d",scores[i]);
		label = gtk_label_new(text);
		g_free(text);
		gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,i+1,i+2);
	}

	return table;
}



static gboolean player_seek_timeout(gpointer data)
{
	rb_player_set_time(player,starting_point);

	g_timeout_add(50,(GSourceFunc)player_set_volume_timeout,NULL);

	return FALSE;
}

static gboolean player_set_volume_timeout(gpointer data)
{
	rb_player_set_volume(player,volume);

	/* this gives people a chance at getting 1000 points (ie, a couple of seconds play before the score starts deducting */
	round_score_start_decrementing_timeout_id = g_timeout_add(2500,(GSourceFunc)round_score_start_decrementing_timeout,NULL);
	round_score_label_update_timeout_id = g_timeout_add(250,(GSourceFunc)round_score_label_update_timeout,NULL);

	return FALSE;
}

static gboolean round_score_start_decrementing_timeout(gpointer data)
{
	/* we've waited a bit, now start decrementing */
	round_score_decrement_timeout_id = g_timeout_add(10,(GSourceFunc)round_score_decrement_timeout,NULL);
	round_entries_desensitize_timeout_id = g_timeout_add(2800,(GSourceFunc)round_entries_desensitize_timeout,NULL);

	round_score_start_decrementing_timeout_id = -1;
	return FALSE;
}


static gboolean round_score_decrement_timeout(gpointer data)
{
	round_score--;

	if (round_score < 0) {
		game_button_clicked_cb(NULL,GINT_TO_POINTER(round_answer-1));		}

	return TRUE;
}

static gboolean round_score_label_update_timeout(gpointer data)
{
	gchar text[5];

	g_sprintf(text,"%d",round_score);
	gtk_label_set_text(round_score_label,text);
	
	return TRUE;
}

static gboolean round_entries_desensitize_timeout(gpointer data)
{
	gint32 i = 0;

	do {
		i = g_random_int_range(0,5);
	} while (round_entries_sensitive[i] == FALSE || i == round_answer);

	round_entries_sensitive[i] = FALSE;
	gtk_widget_set_sensitive(GTK_WIDGET(game_buttons[i]),FALSE);

	return TRUE;
}

static gboolean db_load_progress_bar_pulse_timeout(GtkProgressBar *pbar)
{
	if (db_loaded == TRUE) {
		return FALSE;
	}

	gtk_progress_bar_pulse(pbar);

	return TRUE;
}

static void db_load_complete_cb(RhythmDB *db,gpointer data)
{
	game_model = rhythmdb_query_model_new_empty(db);
	g_signal_connect(G_OBJECT(game_model),"complete",G_CALLBACK(game_model_load_complete_cb),NULL);

	rhythmdb_do_full_query_async(db,GTK_TREE_MODEL(game_model),RHYTHMDB_QUERY_PROP_EQUALS,RHYTHMDB_PROP_TYPE,RHYTHMDB_ENTRY_TYPE_SONG,RHYTHMDB_QUERY_END);

	return;
}

static void game_model_load_complete_cb(RhythmDBQueryModel *model,gpointer data)
{
	num_songs_in_model = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(model),NULL);
	gtk_widget_destroy(db_load_window);
	db_loaded = TRUE;

	new_game();

	return;
}

static void game_button_clicked_cb(GtkWidget *game_button,gpointer data)
{
	gint num;
	gint next_player = (current_player+1)%num_players;
	GtkWidget *dialog;
	GtkBox *vbox;
	GtkWidget *hsep;
	GtkWidget *label;
	GtkWidget *score_widget;
	gchar *text;

	if (round_score_start_decrementing_timeout_id != -1) {
		g_source_remove(round_score_start_decrementing_timeout_id);
	}
	if (round_score_decrement_timeout_id != -1) {
		g_source_remove(round_score_decrement_timeout_id);
	}
	g_source_remove(round_score_label_update_timeout_id);
	g_source_remove(round_entries_desensitize_timeout_id);

	round_entries_desensitize_timeout_id = round_score_start_decrementing_timeout_id = round_score_decrement_timeout_id = round_score_label_update_timeout_id = -1;
	rb_player_pause(player);

	num = GPOINTER_TO_INT(data);

	if (num == round_answer) {
		scores[current_player] += round_score;
	
		text = g_strdup_printf("Correct for %d points.",round_score);
	} else {
		text = g_strdup("Incorrect.");
	}

	dialog = gtk_dialog_new_with_buttons("Scores",GTK_WINDOW(window),GTK_DIALOG_MODAL,GTK_STOCK_OK,GTK_RESPONSE_ACCEPT,NULL);

	vbox = GTK_BOX(GTK_DIALOG(dialog)->vbox);

	label = gtk_label_new(text);
	g_free(text);
	gtk_box_pack_start(vbox,label,FALSE,FALSE,0);

	hsep = gtk_hseparator_new();
	gtk_box_pack_start(vbox,hsep,FALSE,FALSE,6);

	score_widget = make_scores_widget();
	gtk_box_pack_start(vbox,score_widget,TRUE,TRUE,0);

	hsep = gtk_hseparator_new();
	gtk_box_pack_start(vbox,hsep,FALSE,FALSE,6);

	text = g_strdup_printf("It is now Player %d's turn",next_player+1);
	label = gtk_label_new(text);
	g_free(text);
	gtk_box_pack_start(vbox,label,FALSE,FALSE,0);

	gtk_widget_show_all(dialog);

	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);

	play_round();

	return;
}

static void new_game_druid_page_finish_cb(GnomeDruidPage *page,GnomeDruid *druid,gpointer data)
{
	GtkSpinButton *spin_button;
	GSList *radio_button_group;
	GSList *l;
	guint i;
	GtkWidget *druid_window;

	spin_button = GTK_SPIN_BUTTON(g_object_get_data(G_OBJECT(druid),"spin_button"));

	num_players = (gint)gtk_spin_button_get_value(spin_button);
	scores = g_new0(gint,num_players);

	radio_button_group = g_object_get_data(G_OBJECT(druid),"radio_button_group");

	for (l = radio_button_group, i = 0; l; l = l->next, i++) {
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(l->data))) {
			static const MQGameType game_types[] = {MQ_GAME_TITLE,MQ_GAME_ALBUM,MQ_GAME_ARTIST};

			game_type = game_types[i];
		}
	}

	druid_window = g_object_get_data(G_OBJECT(druid),"window");
	gtk_widget_destroy(druid_window);

	round = 0;
	current_player = -1;

	gtk_widget_set_sensitive(window,TRUE);
	gtk_widget_show_all(window);
	first_run = FALSE;

	play_round();

	return;
}

static gboolean new_game_druid_page_cancel_cb(GnomeDruidPage *page,GnomeDruid *druid,gpointer data)
{
	if (first_run) {
		gtk_main_quit();
	}

	return FALSE;
}

static void new_game_cb(GtkWidget *widget,gpointer data)
{
	new_game();

	return;
}

static void scores_cb(GtkWidget *widget,gpointer data)
{
//	g_print("Scores\n");
	
	return;
}

static void quit_cb(GtkWidget *widget,gpointer data)
{
	gtk_main_quit ();
	
	return;
}

static void about_cb(GtkWidget *widget,gpointer data)
{
        static GtkWidget *about;
	GdkPixbuf *pixbuf = NULL;

        const gchar *authors[] = {
		"Charles Schmidt",
		NULL
	};
	const gchar *documenters[] = {
                NULL
        };
	const gchar *translator_credits = _("translator-credits");
	
	if (about) {
                gtk_window_present(GTK_WINDOW(about));

		return;
	}

	{
		gint i = 0;
		while (authors[i] != NULL) { 
			authors[i] = _(authors[i]); 
			i++; 
		}
	}
	{
		gchar *filename = NULL;

		filename = gnome_program_locate_file (NULL,GNOME_FILE_DOMAIN_APP_PIXMAP,("gnome-mq.png"),TRUE, NULL);

		if (filename != NULL) {
			pixbuf = gdk_pixbuf_new_from_file(filename, NULL);
			g_free (filename);
		}
	}

	about = gnome_about_new(_("GNOME Music Quiz"),VERSION,"Copyright \xc2\xa9 2004 Charles Schmidt",_("A Music Quiz game like the television show 'Name That Tune.'"),(const char **)authors,(const char **)documenters,strcmp(translator_credits,"translator-credits") != 0 ? translator_credits : NULL,pixbuf);
	
	if (pixbuf != NULL) {
		g_object_unref(pixbuf);
	}
	
	g_signal_connect(GTK_OBJECT(about),"destroy",GTK_SIGNAL_FUNC(gtk_widget_destroyed),&about);
	gtk_window_set_transient_for(GTK_WINDOW(about),GTK_WINDOW(window));
        gtk_widget_show(about);

	return;
}


gint main(gint argc,gchar **argv)
{
	GnomeClient *client;
	gchar *db_pathname;
	GError *error = NULL;
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *hsep;
	GtkWidget *label;
	GtkWidget *pbar;
	gint i;
	
	gst_init(&argc,&argv);
	gst_control_init(&argc,&argv);

	gnome_score_init("mq");

	bindtextdomain(GETTEXT_PACKAGE,GNOMELOCALEDIR);
	bind_textdomain_codeset(GETTEXT_PACKAGE,"UTF-8");
	textdomain(GETTEXT_PACKAGE);

	gnome_program_init("mq",VERSION,LIBGNOMEUI_MODULE,argc,argv,GNOME_PARAM_APP_DATADIR,DATADIR,NULL);

	gnome_window_icon_set_default_from_file(GNOME_ICONDIR"/gnome-mq.png");

	client = gnome_master_client();
//	g_signal_connect(G_OBJECT(client),"save_yourself",G_CALLBACK(save_yourself_cb),argv[0]);

//	rb_debug_init(TRUE);
	rb_debug_init(FALSE);
	rb_file_helpers_init();
	rb_string_helpers_init();

	db_pathname = g_build_filename(rb_dot_dir(),"rhythmdb.xml",NULL);

	db = rhythmdb_tree_new(db_pathname);
	g_free(db_pathname);
	g_signal_connect(G_OBJECT(db),"load-complete",G_CALLBACK(db_load_complete_cb),NULL);
	rhythmdb_load(db);

	player = rb_player_new(&error);
	if (error) {
/* this should be displayed in an error box somewhere... */
		g_warning("Error creating RBPlayer object: %s",error->message);
		g_error_free(error);
		error = NULL;
	}
	volume = eel_gconf_get_float("/apps/rhythmbox/state/volume");
	if (volume < 0.0) {
		volume = 0.0;
	} else if (volume > 1.0) {
		volume = 1.0;
	}
	rb_player_set_volume(player,volume);

	window = gnome_app_new("mq",_("GNOME Music Quiz"));
	gtk_window_resize(GTK_WINDOW(window),300,250);
	gnome_app_create_menus(GNOME_APP(window),mainmenu);

	g_signal_connect(G_OBJECT(window),"delete_event",G_CALLBACK(quit_cb),NULL);

	vbox = gtk_vbox_new(FALSE,6);
	gtk_container_set_border_width(GTK_CONTAINER(vbox),6);
	gnome_app_set_contents(GNOME_APP(window),vbox);

	hbox = gtk_hbox_new(FALSE,6);
	gtk_box_pack_start(GTK_BOX(vbox),hbox,FALSE,FALSE,0);

	header_label = GTK_LABEL(gtk_label_new(_("GNOME Music Quiz")));
	gtk_box_pack_start(GTK_BOX(hbox),GTK_WIDGET(header_label),FALSE,FALSE,0);

	round_score_label = GTK_LABEL(gtk_label_new("0"));
	gtk_box_pack_end(GTK_BOX(hbox),GTK_WIDGET(round_score_label),FALSE,FALSE,0);

	hsep = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox),hsep,FALSE,FALSE,0);

	for (i = 0; i < 5; i++) {
		gchar *s;

		hbox = gtk_hbox_new(FALSE,6);
		gtk_box_pack_start(GTK_BOX(vbox),hbox,TRUE,TRUE,0);

		s = g_strdup_printf("_%d.",i+1);

		label = gtk_label_new_with_mnemonic(s);
		gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,0);

		game_buttons[i] = GTK_BUTTON(gtk_button_new_with_label(""));
		g_signal_connect(G_OBJECT(game_buttons[i]),"clicked",G_CALLBACK(game_button_clicked_cb),GINT_TO_POINTER(i));
		gtk_button_set_alignment(game_buttons[i],0.0,0.5);
		gtk_label_set_mnemonic_widget(GTK_LABEL(label),GTK_WIDGET(game_buttons[i]));
		gtk_box_pack_start(GTK_BOX(hbox),GTK_WIDGET(game_buttons[i]),TRUE,TRUE,0);

	}
	
	hsep = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox),hsep,FALSE,FALSE,0);

	hbox = gtk_hbox_new(FALSE,6);
	gtk_box_pack_start(GTK_BOX(vbox),hbox,FALSE,FALSE,0);

	score_label = GTK_LABEL(gtk_label_new("Score: 0"));
	gtk_box_pack_start(GTK_BOX(hbox),GTK_WIDGET(score_label),FALSE,FALSE,0);

	round_label = GTK_LABEL(gtk_label_new("Round: 1/25"));
	gtk_box_pack_end(GTK_BOX(hbox),GTK_WIDGET(round_label),FALSE,FALSE,0);

	gtk_widget_set_sensitive(window,FALSE);

	db_load_window = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(db_load_window),_("Loading Music Information"));

	vbox = GTK_DIALOG(db_load_window)->vbox;

	label = gtk_label_new(_("Loading Music Information"));
	gtk_box_pack_start(GTK_BOX(vbox),label,FALSE,FALSE,0);

	pbar = gtk_progress_bar_new();
	gtk_box_pack_start(GTK_BOX(vbox),pbar,FALSE,FALSE,0);

	gtk_widget_show_all(db_load_window);

	g_timeout_add(1000,(GSourceFunc)db_load_progress_bar_pulse_timeout,pbar);

        gtk_main ();

	rb_file_helpers_shutdown();
	rb_string_helpers_shutdown();

	return 0;
}




